/*
 * Copyright (c) 2012 - 2013 Alexandre Tuleu
 *
 * This file is part of biorob-cpp.
 *
 *   biorob-cpp is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as
 *   published by the Free Software Foundation, either version 3 of the
 *   License, or (at your option) any later version.
 *
 *   biorob-cpp is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of both the GNU General Public
 *   License AND GNU Lesser General Public License along with
 *   biorob-cpp.  If not, see <http://www.gnu.org/licenses/>.
 */


#include "DummyPlugin.h"
#include <biorob-cpp/plugin/PluginLoader.cpp>
#include <iostream>
#include <exception>


int main(int argc, char ** argv){
	std::string choosen;
	if(argc >= 2){
		choosen.append(argv[1]);
	}

	PluginLoader<DummyPlugin> loader;

	//we tell the loader that our plugin was built with the CMake
	// MODULE option

	loader.SetHint((PluginLoader<DummyPlugin>::LoadingHint)
			(  PluginLoader<DummyPlugin>::ResolveAllSymbols
			 | PluginLoader<DummyPlugin>::IsCMakeModuleObject));

	//for example reason, if user did not choose a file, we just try
	//all of them
	if(choosen.empty()){
		//you don't have to provide the exact name, it will be infered
		// by the module like cmake does.
		loader.AppendVariant("plugin-a");
		loader.AppendVariant("plugin-b");
	} else {
		loader.SetFilename(choosen);
	}

	try {
		PluginLoader<DummyPlugin>::PluginPtr plugin = loader.LoadPlugin();
		plugin->Print(std::cout);
	} catch (const std::exception & e) {
		std::cerr << e.what() << std::endl;
		return 1;
	}

	return 0;
}
