/*
 * Copyright (c) 2012 - 2013 Alexandre Tuleu
 *
 * This file is part of biorob-cpp.
 *
 *   biorob-cpp is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as
 *   published by the Free Software Foundation, either version 3 of the
 *   License, or (at your option) any later version.
 *
 *   biorob-cpp is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of both the GNU General Public
 *   License AND GNU Lesser General Public License along with
 *   biorob-cpp.  If not, see <http://www.gnu.org/licenses/>.
 */


/*
 * Copyright (c) 2012  Alexandre Tuleu
 *
 * This file is part of biorob-cpp.
 *
 *   biorob-cpp is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as
 *   published by the Free Software Foundation, either version 3 of the
 *   License, or (at your option) any later version.
 *
 *   biorob-cpp is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of both the GNU General Public
 *   License AND GNU Lesser General Public License along with
 *   PROJECT.  If not, see <http://www.gnu.org/licenses/>.
 */


#ifndef BIOROB_CLI_PROMPT_H_
#define BIOROB_CLI_PROMPT_H_

#include <string>
#include <map>
#include <set>
#include <vector>

#include "Widget.h"
#include "Command.h"

namespace cli {
class Prompt : public Widget{
 public :
	Prompt(const Widget::Def & def);
	virtual ~Prompt();

	const std::string & GetLine(const std::string & value);

	void AddCompletion(const std::string & c);
	void RemoveCompletion(const std::string & c);

	void BindKey(int c, KeyCallback::Ptr callback);
	void UnBindKey(int c);

	void WindowInit();

 private :
	typedef void (Prompt::* CallbackPtr )(void);
	typedef std::map<int,CallbackPtr> CallbackByKey;
	typedef std::map<int,KeyCallback::Ptr> KeyCallbackByKey;
	typedef std::vector<std::string>  History;
	typedef std::set<std::string>     CompletionSet;
	typedef std::pair<CompletionSet::const_iterator,CompletionSet::const_iterator> CompletionResult;

	void RefreshLine();

	void IncrementCursor();
	void DecrementCursor();
	void InsertCharAtCursor(int c,bool escape = false);

	void MoveCursorToNextWord();
	void MoveCursorToPrevWord();

	void MoveCursorAtBegin();
	void MoveCursorAtEnd();
	void MoveCursorAtInsertionPoint();


	void EraseAtCursor();
	void EraseToEOL();


	void IncrementDrawingCursor();

	void Ignore();

	void HistoryUp();
	void HistoryDown();

	void Complete();
	void GetCurrentWord(std::string & w,
	                    std::string::iterator & start,
	                    std::string::iterator & end);
	CompletionResult GetCompletionResult(const std::string & value,
	                                     unsigned int & size);

	void PrintCompletion(CompletionResult & result);

	void LineToWinPos(size_t linePos, int & x, int & y);


	size_t d_starty;
	std::string d_prompt;
	History     d_history;
	History::const_iterator d_historyPtr;
	std::string d_result,d_backupResult;
	std::string::iterator d_curp;
	CallbackByKey   d_callbacks;
	KeyCallbackByKey d_externalCallbacks;
	bool d_currentCharRepeated;

	CompletionSet d_availableCompletions;

};
}//namespace cli


#endif //BIOROB_CLI_PROMPT_H_
