/*
 * Copyright (c) 2012 - 2013 Alexandre Tuleu
 *
 * This file is part of biorob-cpp.
 *
 *   biorob-cpp is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as
 *   published by the Free Software Foundation, either version 3 of the
 *   License, or (at your option) any later version.
 *
 *   biorob-cpp is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of both the GNU General Public
 *   License AND GNU Lesser General Public License along with
 *   biorob-cpp.  If not, see <http://www.gnu.org/licenses/>.
 */


/**
 * \file base/AllInstanceExecuter.h
 *
 *  \date Dec 20, 2010
 *  \author Alexandre Tuleu
 */

#ifndef BIOROB_CPP_BASE_ALL_INSTANCE_EXECUTER_H_
#define BIOROB_CPP_BASE_ALL_INSTANCE_EXECUTER_H_

#include <set>

/**
 *
 */
template <typename T>
class AllInstanceExecuter {
public :
	virtual ~AllInstanceExecuter();
	template <typename U> static void executeForAll(U & functor);
protected :
	AllInstanceExecuter(T * me);
private :
	static std::set<T*> s_allCreatedObjects;
	T* d_me;

};

template <typename T>
inline AllInstanceExecuter<T>::AllInstanceExecuter(T* me) : d_me(me){
	s_allCreatedObjects.insert(me);
}

template <typename T>
AllInstanceExecuter<T>::~AllInstanceExecuter(){
	s_allCreatedObjects.erase(d_me);
}

template <typename T>
template <typename U>
inline void AllInstanceExecuter<T>::executeForAll(U & functor){
	for(typename std::set<T*>::iterator object = s_allCreatedObjects.begin();
	    object != s_allCreatedObjects.end();
	    ++object){
		functor(*(*object));
	}
}

template <typename T>
std::set<T*> AllInstanceExecuter<T>::s_allCreatedObjects;

#endif // LIBATULEU_ALLINSTANCEEXECUTER_H_
